# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['edfi_lms_file_utils']

package_data = \
{'': ['*']}

install_requires = \
['numpy==1.19.3', 'pandas>=1.1.4,<2.0.0']

setup_kwargs = {
    'name': 'edfi-lms-file-utils',
    'version': '1.0.0a6',
    'description': 'Utilities to facilitate use of the filesystem created by Ed-Fi LMS Extractors',
    'long_description': '# file-utils\n\nContains a set of functions to help work with the files generated by the LMS\nextractors. Does not contain any stand-alone scripts for execution.\n\nFor more information, and to see them in action, open the [filesystem-tutorial\nnotebook](../notebooks/filesystem-tutorial.ipynb)\n',
    'author': 'Ed-Fi Alliance, LLC and contributors',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://techdocs.ed-fi.org/display/EDFITOOLS/LMS+Toolkit',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
